% REPRODUCING EXPERIMENT 3: mode = exp(lambda_i)
% Active correlation learning

% Only 1 change made: replace the log-likelihood score by correlation
% See ClearnCORR.c for details

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
scale = .7;  % scale of Gabors, length = 17 pixels
norient = 15;  % number of orientations
epsilon = .1;  % allowed correlation between selected Gabors
T = 0.;        % threshold for responses
Totalsketch = 40;  % allowed number of selected Gabors   
Upperbound = 14.; % saturation value
sub = 2;    % subsampling rate for Gabor
Lrange = 3; % allowed shifting in location (times sub pixels)
Orange = 1; % allowed shifting in orientation
%%% Note: Lrange*sub is the pixels allowed to shift along normal
%%%       Lrange*sub should be the same in learning and testing!
LIK = 0.; SHUTUP = 1.; 

% TRAINING EXAMPLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ==================================================================
%  input images from a folder
% ==================================================================
posFolder = 'headShoulder_trainpos85_127';  % image input folder
Iname = dir([posFolder '\*.jpg']);
destFolder = '.\abCORR_headShoulder'; % output folder
if ~exist(destFolder,'dir')
    mkdir(destFolder);
end

nimage = size(Iname, 1);
Isize = zeros(nimage, 2);
for (i = 1 : nimage)
    I{i} = imresize(double(rgb2gray(imread([posFolder '\' Iname(i).name]))), 1);
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for (i = 1 : nimage)
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the training images the same



% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
C = corr(allfilter, epsilon);    % generate the inhibition maps
allfiltered = applyfilter(I, allfilter, T);  % filter training images

% WHITENING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the following three lines are the only things that need to be added
load 'nlf'; 
mex Ctransform.c; 
Ctransform(nimage, norient, allfiltered, sx, sy, nlf, bnf, bsf);


% VARIABLES FOR MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % average response of selected Gabor
sym = zeros(size(I{1}));  % symbolic plot of selected Gabor
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for (i = 1 : nimage)
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, 1);  % total scores for testing images
gain = zeros(Totalsketch, 1);  % coding gain for selected Gabor

responseVec = zeros(nimage,length(Mi));

binnum = 50; % number of bins 
% MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
mex Clearn.c;   % compile C code
Clearn(nimage, norient, allfiltered, C, h, sx, sy, T, Mi, Mx, My, Mm, allsymbol(1, :), sym, Asym, Lrange, Orange, sub, Totalsketch, Upperbound, tss, gain, responseVec, LIK, SHUTUP, binnum);


% RESULTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
K = nimage;
[tsssort, ind] = sort(tss); 

% save sketch images and results
towrite = -(sym); 
towrite = uint8(255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));
imwrite(towrite,[destFolder '\learnedTemplate.jpg']);
for i = 1 : nimage
    towrite = -(Asym{i});
    towrite = uint8(255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));
    imwrite(towrite,[destFolder '\sym_' Iname(i).name],'jpg');
end

save([destFolder '\model.mat'],'sym','allsymbol','T','Mi','Mx','Mm','My','C','h','Lrange','Orange','sub','scale','norient','epsilon','allfilter','allsymbol','Asym','gain','Upperbound','Totalsketch','scale','allfilter','responseVec');