function tss = ABtest_one(modelFolder, srcFolder,destFolder)
% modelFolder : folder that contains trained model file 'model.mat'
% srcFolder : image input folder
% destFolder : destination folder

load([modelFolder '\model.mat']); % load trained shared-sketch model
Iname = dir([srcFolder '\*.jpg']); % image file names from input folder
doNotSave = false;
if size(destFolder,1)==0   % if <destFolder> is null, do not save result
    doNotSave = true;
else if ~exist(destFolder,'dir')
        mkdir(destFolder);
    end
end

nimage = size(Iname, 1); % # images
Isize = zeros(nimage, 2);
for i = 1 : nimage
    tmpIm = imread([srcFolder '\' Iname(i).name]);
    if size(tmpIm,3) == 3
        tmpIm = rgb2gray(tmpIm);
    end
    I{i} = imresize(double(tmpIm), 1); 
    Isize(i, :) = size(I{i});
end
sx = min(Isize(:, 1)); sy = min(Isize(:, 2)); 
for (i = 1 : nimage)
    I{i} = I{i}(1:sx, 1:sy); 
end   % make the sizes of the testing images the same

[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
% filter the testing images 
allfiltered = applyfilter(I, allfilter, T);  % filter testing images

% WHITENING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% the following three lines are the only things that need to be added
load 'nlf'; 
mex Ctransform.c; 
Ctransform(nimage, norient, allfiltered, sx, sy, nlf, bnf, bsf);

% prepare the spaces for C code
Asym = cell(1, nimage);   % symbolic plot for each image with active Gabor
for (i = 1 : nimage)
    Asym{i} = zeros(size(I{1}));  
end
tss = zeros(nimage, 1);  % total scores for testing images

responseVec = zeros(nimage,length(Mi)); % feature responses (a 2D matrix)

% pass on the pointers to C for heavy computation
mex Ctesting.c
Ctesting(nimage, norient, allfiltered, C, h, sx, sy, T, Mi, Mx, My, Mm, allsymbol(1, :), Asym, Lrange, Orange, sub, Totalsketch, Upperbound, tss, responseVec);

% save sketches into image files, and save results to .mat file
if ~doNotSave
    for i = 1 : nimage
        towrite = -(Asym{i});
        towrite = uint8(255 * (towrite-min(towrite(:)))/(max(towrite(:))-min(towrite(:))));
        imwrite(towrite,[destFolder '\sym_' Iname(i).name],'jpg');
    end
    save([destFolder '\testResult.mat'],'tss','responseVec','Asym','allfiltered');
end

